@@ -2,7 +2,7 @@
 #
 # Packager - A class for writing the Excel XLSX Worksheet file.
 #
-# Copyright 2013-2019, John McNamara, jmcnamara@cpan.org
+# Copyright 2013-2021, John McNamara, jmcnamara@cpan.org
 #
 
 # Standard packages.
@@ -12,7 +12,7 @@ import tempfile
 from shutil import copy
 
 from .compatibility import StringIO
-from .compatibility import BytesIO
+from io import BytesIO
 
 # Package imports.
 from .app import App
@@ -129,6 +129,9 @@ class Packager(object):
 
     def _create_package(self):
         # Write the xml files that make up the XLSX OPC package.
+        self._write_content_types_file()
+        self._write_root_rels_file()
+        self._write_workbook_rels_file()
         self._write_worksheet_files()
         self._write_chartsheet_files()
         self._write_workbook_file()
@@ -138,19 +141,16 @@ class Packager(object):
         self._write_comment_files()
         self._write_table_files()
         self._write_shared_strings_file()
-        self._write_app_file()
-        self._write_core_file()
-        self._write_custom_file()
-        self._write_content_types_file()
         self._write_styles_file()
+        self._write_custom_file()
         self._write_theme_file()
-        self._write_root_rels_file()
-        self._write_workbook_rels_file()
         self._write_worksheet_rels_files()
         self._write_chartsheet_rels_files()
         self._write_drawing_rels_files()
         self._add_image_files()
         self._add_vba_project()
+        self._write_core_file()
+        self._write_app_file()
 
         return self.filenames
 
@@ -318,6 +318,7 @@ class Packager(object):
             app._add_part_name(named_range)
 
         app._set_properties(properties)
+        app.doc_security = self.workbook.read_only
 
         app._set_xml_writer(self._filename('docProps/app.xml'))
         app._assemble_xml_file()
@@ -348,6 +349,8 @@ class Packager(object):
         content = ContentTypes()
         content._add_image_types(self.workbook.image_types)
 
+        self._get_table_count()
+
         worksheet_index = 1
         chartsheet_index = 1
         for worksheet in self.workbook.worksheets():
@@ -398,6 +401,7 @@ class Packager(object):
         fill_count = self.workbook.fill_count
         custom_colors = self.workbook.custom_colors
         dxf_formats = self.workbook.dxf_formats
+        has_comments = self.workbook.has_comments
 
         styles = Styles()
         styles._set_style_properties([
@@ -408,7 +412,8 @@ class Packager(object):
             border_count,
             fill_count,
             custom_colors,
-            dxf_formats])
+            dxf_formats,
+            has_comments])
 
         styles._set_xml_writer(self._filename('xl/styles.xml'))
         styles._assemble_xml_file()
@@ -435,9 +440,14 @@ class Packager(object):
                                                      + str(index) + '.xml'))
                 table._set_properties(table_props)
                 table._assemble_xml_file()
-                self.table_count += 1
                 index += 1
 
+    def _get_table_count(self):
+        # Count the table files. Required for the [Content_Types] file.
+        for worksheet in self.workbook.worksheets():
+            for table_props in worksheet.tables:
+                self.table_count += 1
+
     def _write_root_rels_file(self):
         # Write the _rels/.rels xml file.
         rels = Relationships()
