@@ -2,7 +2,7 @@
 #
 # Workbook - A class for writing the Excel XLSX Workbook file.
 #
-# Copyright 2013-2020, John McNamara, jmcnamara@cpan.org
+# Copyright 2013-2021, John McNamara, jmcnamara@cpan.org
 #
 
 # Standard packages.
@@ -140,6 +140,7 @@ class Workbook(xmlwriter.XMLwriter):
         self.calc_on_load = True
         self.calc_id = 124519
         self.has_comments = False
+        self.read_only = 0
 
         # We can't do 'constant_memory' mode while doing 'in_memory' mode.
         if self.in_memory:
@@ -325,6 +326,12 @@ class Workbook(xmlwriter.XMLwriter):
                                     "Use workbook.use_zip64().")
 
             self.fileclosed = True
+
+            # Ensure all constant_memory temp files are closed.
+            if self.constant_memory:
+                for worksheet in self.worksheets():
+                    worksheet._opt_close()
+
         else:
             warn("Calling close() on already closed file.")
 
@@ -586,6 +593,19 @@ class Workbook(xmlwriter.XMLwriter):
         else:
             self.vba_codename = 'ThisWorkbook'
 
+    def read_only_recommended(self):
+        """
+        Set the Excel "Read-only recommended" option when saving a file.
+
+        Args:
+            None.
+
+        Returns:
+            Nothing.
+
+        """
+        self.read_only = 2
+
     ###########################################################################
     #
     # Private API.
@@ -607,6 +627,9 @@ class Workbook(xmlwriter.XMLwriter):
         # Write the fileVersion element.
         self._write_file_version()
 
+        # Write the fileSharing element.
+        self._write_file_sharing()
+
         # Write the workbookPr element.
         self._write_workbook_pr()
 
@@ -1296,7 +1319,7 @@ class Workbook(xmlwriter.XMLwriter):
         # Look for numbers rather than strings for Python 2.6/3 compatibility.
         marker_ihdr = 0x49484452  # IHDR
         marker_phys = 0x70485973  # pHYs
-        marker_iend = 0X49454E44  # IEND
+        marker_iend = 0x49454E44  # IEND
 
         # Search through the image data to read the height and width in the
         # IHDR element. Also read the DPI in the pHYs element.
@@ -1696,6 +1719,15 @@ class Workbook(xmlwriter.XMLwriter):
 
         self._xml_empty_tag('fileVersion', attributes)
 
+    def _write_file_sharing(self):
+        # Write the <fileSharing> element.
+        if self.read_only == 0:
+            return
+
+        attributes = [('readOnlyRecommended', 1)]
+
+        self._xml_empty_tag('fileSharing', attributes)
+
     def _write_workbook_pr(self):
         # Write <workbookPr> element.
         default_theme_version = 124226
